@@ -29,7 +29,7 @@
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 
-#include "shell/list-box-helper.h"
+#include "list-box-helper.h"
 #include "cc-common-language.h"
 #include "cc-util.h"
 
