@@ -19,7 +19,7 @@
  */
 
 #include "cc-search-locations-dialog.h"
-#include "shell/list-box-helper.h"
+#include "list-box-helper.h"
 
 #include <glib/gi18n.h>
 
@@ -370,10 +370,15 @@ switch_tracker_get_mapping (GValue *value,
   for (idx = 0; locations[idx] != NULL; idx++)
     {
       location = g_file_new_for_path (path_from_tracker_dir(locations[idx]));
-      if ((found = g_file_equal (location, place->location)))
+      found = g_file_equal (location, place->location);
+      g_object_unref (location);
+
+      if (found)
         break;
     }
 
+  g_free (locations);
+
   g_value_set_boolean (value, found);
   return TRUE;
 }
